#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "BSTree.h"

// Returns the subtree height - This is equal to the maximum of the two children heights
int subtreeHeight(BSTreeNode *node)
{
	int height = 0;
	assert(node != NULL);

	if ((node->leftChild != NULL) && (node->rightChild != NULL))
		height = MAX(node->leftChild->height, node->rightChild->height);
	else if ((node->leftChild == NULL) && (node->rightChild == NULL))
		height = 0;
	else if (node->leftChild != NULL)
		height = node->leftChild->height;
	else
		height = node->rightChild->height;

	return height;
}

// Destroy a subtree
void deleteSubtree(BSTreeNode *node)
{
	assert(node != NULL);
	// First destroy the children
	if (node->leftChild != NULL)
		deleteSubtree(node->leftChild);
	if (node->rightChild != NULL)
		deleteSubtree(node->rightChild);
	// Delete the node
	free(node);
}

// Create a new node
BSTreeNode *createNode(Type value)
{
	BSTreeNode *node = (BSTreeNode *)malloc(sizeof(BSTreeNode));

	if (node == NULL) {
		fprintf(stderr, "Memory Allocation Error! Exiting!\n");
		exit(1);
	}
	node->data = value;
	node->height = 0;
	node->leftChild = node->rightChild = NULL;
	node->father = NULL;

	return node;
}

// Print the subtree in infix order
void printNodeInfix(BSTreeNode *node)
{
	if (node == NULL)
		return;

	if (node->leftChild)
		printNodeInfix(node->leftChild);

	fprintf(stdout, TYPE_MODIFIER" ", node->data);

	if (node->rightChild)
		printNodeInfix(node->rightChild);
}

// Print the subtree in prefix order
void printNodePrefix(BSTreeNode *node)
{
	if (node == NULL)
		return;

	fprintf(stdout, TYPE_MODIFIER" ", node->data);

	if (node->leftChild)
		printNodePrefix(node->leftChild);
	if (node->rightChild)
		printNodePrefix(node->rightChild);
}

// Print the subtree in postfix order
void printNodePostfix(BSTreeNode *node)
{
	if (node == NULL)
		return;

	if (node->leftChild)
		printNodePostfix(node->leftChild);
	if (node->rightChild)
		printNodePostfix(node->rightChild);
	
	fprintf(stdout, TYPE_MODIFIER" ", node->data);
}

// Insert a node into the Binary Search Tree
void insertNode(BSTreeNode *root, BSTreeNode *node)
{
	assert(root != NULL);
	assert(node != NULL);

	///////////////////////////////////////////////////
	////////////   INSERT CODE HERE  //////////////////
	///////////////////////////////////////////////////
}

// Find a node containing a data value
BSTreeNode *findNode(BSTreeNode *node, Type value)
{
	///////////////////////////////////////////////////
	////////////   INSERT CODE HERE  //////////////////
	///////////////////////////////////////////////////
	return NULL;
}

// Delete the tree
void deleteTree(BSTree *tree)
{
	assert(tree != NULL);

	if (tree->root != NULL)
		deleteSubtree(tree->root);
	free(tree);
}

// Create a new tree with no root
BSTree *createTree(void)
{
	BSTree *tree = NULL;

	tree = (BSTree *)malloc(sizeof(BSTree));
	if (tree == NULL) {
		fprintf(stderr, "Memory Allocation Error! Exiting!\n");
		exit(1);
	}

	tree->root = NULL;
	return tree;
}

// Insert the data into the node
void insertData(BSTree *tree, Type value)
{
	BSTreeNode *newNode = NULL;

	assert(tree != NULL);

	newNode = createNode(value);

#ifdef _DEBUG
	fprintf(stdout, "\nInserting Value \'"TYPE_MODIFIER"\'\n", value);
	fprintf(stdout, "------------------\n", value);
#endif

	///////////////////////////////////////////////////
	////////////   INSERT CODE HERE  //////////////////
	///////////////////////////////////////////////////

}

// Print the tree in multiple visit orders
void printTree(BSTree *tree)
{
	assert(tree != NULL);

	fprintf(stdout, "Tree contents (infix): ");
	printNodeInfix(tree->root);
	fprintf(stdout,  "\n");

	fprintf(stdout, "Tree contents (prefix): ");
	printNodePrefix(tree->root);
	fprintf(stdout,  "\n");

	fprintf(stdout, "Tree contents (postfix): ");
	printNodePostfix(tree->root);
	fprintf(stdout,  "\n");
}

// Find a node that contains a data value
BSTreeNode *findData(BSTree *tree, Type value)
{
	BSTreeNode *node = NULL;
	///////////////////////////////////////////////////
	////////////   INSERT CODE HERE  //////////////////
	///////////////////////////////////////////////////

	return node;
}

// Checks if a data value exists in the tree
BOOL dataExists(BSTree *tree, Type value)
{
	assert(tree != NULL);
	if (findData(tree, value) != NULL)
		return TRUE;
	else
		return FALSE;
}
